 --Best = 13750

local M = {}

local helper = require('scenario/scenariohelper')
local scenarioTime = 0
local start = false
local reset = true
local severity = 1
local ballPoints = 0
local ballsPotted = 0
local eventArea = 0
local windTime = 0.01
local alpha = 0
local vehicleObjects = {}
local flip = false
local timeUntilNextStorm = 10
local stormPoints = 0
local stationaryTime = 0
local file = require('scenario/fileHandler')

local function onRaceStart()
	if not start then 
		start = true 
		TorqueScript.eval([[
			triangle_1.position = "10000 0 0";
			triangle_2.position = "10000 0 0";
			triangle_3.position = "10000 0 0";
		]])
		be:reloadStaticCollision();
	end
end

local function success(addMsg)
	local medal, nextMedal
	
	score = ballPoints+stormPoints

	if score < 3000 then 
		medal = 'None'
		nextMedal = 'Bronze(3000)'
	elseif score >= 3000 and score < 4000 then 
		medal = 'Bronze'
		nextMedal = 'Silver(4000)'
	elseif score >= 4000 and score < 5000 then 
		medal = 'Silver'
		nextMedal = 'Gold(5000)'
	elseif score >= 5000 and score < 6000 then 
		medal = 'Gold'
		nextMedal = 'Platinum(6000)'
	elseif score >= 6000 and score < 8750 then 
		medal = 'Platinum'
		nextMedal = 'Diamond(8750)'
	elseif score >= 8750 then 
		medal = 'Diamond'
		nextMedal = 'n/a'
	else
		medal = 'Broken'
		nextMedal = 'Also Broken'
	end
	
	file.setValueToSave(22, file.getMedalAsNumber(medal), 1)
	
    local result = {msg = addMsg..' Well Done! Your Score: '..ballPoints..'+'..stormPoints..' = '..score..' | You Achieved: '..medal..' | Next Medal: '..nextMedal..''}
	scenario_scenarios.finish(result)
end

local function fail(reason)
	scenario_scenarios.finish({failed = reason})
end

local function resetRandom(vehicle)
	x, y = math.random(-609, -519), math.random(-1342, -1182)
	TorqueScript.eval(vehicle..'.position = "'..x..' '..y..' 34";')
	TorqueScript.eval(vehicle..'.rotation = "1 0 0 0"; ')
end

local function onBeamNGTrigger(data)
	if (data.triggerName == 'pocket1' or data.triggerName == 'pocket2' or data.triggerName == 'pocket3' or data.triggerName == 'pocket4' or data.triggerName == 'pocket5' or data.triggerName == 'pocket6') and data.event == 'enter' then 
		if data.subjectName ~= 'player' then
			ballPoints = ballPoints+250
			ballsPotted = ballsPotted+1
			resetRandom(data.subjectName)
		else TorqueScript.eval(data.subjectName..'.position = "-559.627 -1212.56 30.5";')
		end
	elseif string.find(data.triggerName, "outside") then TorqueScript.eval(data.subjectName..'.position = "-559.627 -1212.56 30.5";')
	elseif string.find(data.triggerName, "area") then
		local vehicleObject = scenetree.findObject(data.subjectName)
		if windTime < 10 and windTime ~= 0.01 and tonumber(string.match(data.triggerName, "%d+")) == eventArea then
			if tonumber(string.match(data.triggerName, "%d+")) == eventArea then				
				local windSpeed
				if data.subjectName == 'player' then windSpeed = 75*severity
				else windSpeed = 20+(5*(severity-1))
				end
				
				if eventArea <= 3 then vehicleObject:queueLuaCommand('obj:setWind(0, '..windSpeed..', 0)') end
			else vehicleObject:queueLuaCommand('obj:setWind(0, 0, 0)')
			end
		else vehicleObject:queueLuaCommand('obj:setWind(0, 0, 0)')
		end
	end
end

local function getInstanceColour()
	if severity == 1     then return 0,   255, 0
	elseif severity == 2 then return 255, 255, 0
	elseif severity == 3 then return 255, 174, 0
	elseif severity == 4 then return 255, 0,   0
	elseif severity == 5 then return 255, 255, 255
	else return 0, 0, 0
	end
end

local function flashArea()
	if alpha > 2 or alpha < 0 then flip = not flip end
	if flip then alpha = alpha-0.1
	else alpha = alpha+0.1
	end
	
	local r, g, b = getInstanceColour()
	
	TorqueScript.eval('area'..eventArea..'.instanceColor = "'..r..' '..g..' '..b..' '..alpha..'";')
end

local function onPreRender(dt)
	if reset then
		TorqueScript.eval([[
			triangle_1.position = "-567.786926 -1213.0249 29.0229187";
			triangle_2.position = "-559.72052 -1213.27441 29.0229187";
			triangle_3.position = "-563.669373 -1206.63586 29.0327263";
			area1.position = "10000 0 0";
			area2.position = "10000 0 0";
			area3.position = "10000 0 0";
		]])
		be:reloadStaticCollision();
		reset = false
	end
	
	if windTime > 10 then flashArea()
	elseif alpha ~= 2 then --Forces an alpha of 2 after flashing has occurred
		alpha = 2
		flashArea()
	end
end

local function updateWaypoint(show)
	if eventArea == 1 then 
		if show then TorqueScript.eval('area'..eventArea..'.position = "-523.622681 -1137.88367 44.4875488";')
		else TorqueScript.eval('area'..eventArea..'.position = "10000 0 0";')
		end
	elseif eventArea == 2 then 
		if show then TorqueScript.eval('area'..eventArea..'.position = "-563.471436 -1137.61584 44.7102432";')
		else TorqueScript.eval('area'..eventArea..'.position = "10000 0 0";')
		end
	elseif eventArea == 3 then 
		if show then TorqueScript.eval('area'..eventArea..'.position = "-606.130188 -1137.61584 44.7102432";')
		else TorqueScript.eval('area'..eventArea..'.position = "10000 0 0";')
		end
	end
end

local function getWarningTime()
	if 15-math.floor(ballsPotted/12) >= 13 then return 15-math.floor(ballsPotted/12)
	else return 13
	end
end

local function activateWarning()
	windTime = getWarningTime()
	eventArea = math.random(1, 3)
	helper.flashUiMessage('Category '..severity..' Storm Forming! ', 3)
	updateWaypoint(true)
	timeUntilNextStorm = -1
end

local function selectSeverity()
	if ballsPotted >= 25 then severity = 5
	else
		local rand = math.random(0, 100)
		if rand < 25-(ballsPotted*2) then severity = 1
		elseif rand < 50-(ballsPotted*2) then severity = 2
		elseif rand < 75-(ballsPotted*2) then severity = 3
		elseif rand < 90-(ballsPotted*2) then severity = 4
		else severity = 5
		end
	end
end

local function resetWind()
	scenetree.findObject('player'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball1'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball2'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball3'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball4'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball5'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball6'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball7'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball8'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball9'):queueLuaCommand('obj:setWind(0, 0, 0)')
	scenetree.findObject('ball10'):queueLuaCommand('obj:setWind(0, 0, 0)')
end

local function getFrequency()
	if 15-ballsPotted > 0 then return 15-ballsPotted
	else return 0
	end
end

local function resetVars()
	windTime = 0.01
	alpha = 0
	flip = false
	stormPoints = stormPoints+(10*severity)
	updateWaypoint(false)
	selectSeverity()
	timeUntilNextStorm = getFrequency()
	resetWind()
end

local function timeOut(rtt)
	stationaryTime = stationaryTime+rtt
	if stationaryTime >= 5 then return true
	else return false
	end
end

local function onRaceTick(raceTickTime)
	if start then 
		scenarioTime = scenarioTime+raceTickTime
		if timeUntilNextStorm > 0 then timeUntilNextStorm = timeUntilNextStorm-raceTickTime end
		if windTime > 0.01 then windTime = windTime-raceTickTime end		
	end

	if windTime == 0 then resetVars()
	elseif timeUntilNextStorm == 0 then activateWarning()
	end
	
	local playerData = map.objects[map.objectNames['player']]
	if playerData.damage >= 225000 then success('Your car reached critical damage.')
	elseif playerData.vel:length() <= 0.5 then 
		if timeOut(raceTickTime) then success('Your car was stationary for too long.') end
	elseif stationaryTime > 0 then stationaryTime = 0
	end
	
	ui_message('Balls Potted: '..ballsPotted, 1, 'A')
	ui_message('Damage: '..math.floor((playerData.damage/225000)*100)..'%', 1, 'B')
	ui_message('Total Points: '..ballPoints+stormPoints, 1, 'C')
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onBeamNGTrigger = onBeamNGTrigger
M.onPreRender = onPreRender
return M