--Best = 

local M = {}

local helper = require('scenario/scenariohelper')
local scenarioTimeOut = 0
local start = false
local reset = true
local finalScore = 0
local rotation = 0
local ballsPotted = {}
local ballsLost = 0
local file = require('scenario/fileHandler')

local function onRaceStart()
	if not start then 
		start = true 
		TorqueScript.eval([[
			triangle_1.position = "10000 0 0";
			triangle_2.position = "10000 0 0";
			triangle_3.position = "10000 0 0";
		]])
		be:reloadStaticCollision();
	end
end

local function success(score, pointsGained, pointsLost)
	local medal, nextMedal

	if score < 400 then 
		medal = 'None'
		nextMedal = 'Bronze(400)'
	elseif score >= 400 and score < 500 then 
		medal = 'Bronze'
		nextMedal = 'Silver(500)'
	elseif score >= 500 and score < 700 then 
		medal = 'Silver'
		nextMedal = 'Gold(700)'
	elseif score >= 700 and score < 800 then 
		medal = 'Gold'
		nextMedal = 'Platinum(800)'
	elseif score >= 800 and score < 1000 then 
		medal = 'Platinum'
		nextMedal = 'Diamond(1000)'
	elseif score >= 1000 then 
		medal = 'Diamond'
		nextMedal = 'n/a'
	else
		medal = 'Broken'
		nextMedal = 'Also Broken'
	end
	
	file.setValueToSave(23, file.getMedalAsNumber(medal), 1)
	
    local result = {msg = 'Well Done! Your Score: '..pointsGained..'-'..pointsLost..' ='..score..' | You Achieved: '..medal..' | Next Medal: '..nextMedal..''}
	scenario_scenarios.finish(result)
end

local function fail(reason)
  scenario_scenarios.finish({failed = reason})
end

local function onBeamNGTrigger(data)
	if string.find(data.triggerName, 'saw') then
		if data.subjectName == 'player' then
			if data.event == 'enter' then
				local number = tonumber(string.match(data.triggerName, "%d+"))
				local cmd = ''
				if number == 3 or number == 5 then cmd = 'obj:setWind(0, '..(-300-number*50)..', 0)'
				else cmd = 'obj:setWind(0, '..(300+number*50)..', 0)'
				end
				scenetree.findObject('player'):queueLuaCommand(cmd)
			else scenetree.findObject('player'):queueLuaCommand('obj:setWind(0, 0, 0)')
			end
		elseif data.event == 'enter' then
			TorqueScript.eval(''..data.subjectName..'.position = "10000 0 -10";')
			helper.flashUiMessage('A Ball Has Been Destroyed!', 3)
			ballsLost = ballsLost+1
		end
	elseif data.subjectName == 'player' then
		if data.triggerName == 'outside' then fail('Foul: You flew off the table!')
		else fail('Foul: You drove into a pocket!')
		end
	elseif (data.triggerName == 'pocket1' or data.triggerName == 'pocket2' or data.triggerName == 'pocket3' or data.triggerName == 'pocket4' or data.triggerName == 'pocket5' or data.triggerName == 'pocket6') and data.subjectName ~= 'player' and data.event == 'enter' then 
		finalScore = finalScore+100 
		table.insert(ballsPotted, data.subjectName)
	elseif data.triggerName == 'outside' then
		local potted = false
		
		for key, value in ipairs(ballsPotted) do
			if value == data.subjectName then potted = true end 
		end
		
		if not potted then fail('Foul: A ball flew off the table!') end
	end
end

local function onPreRender(dt)
	if reset then
		TorqueScript.eval([[
			triangle_1.position = "-563.864563 -1213.0249 29.0229187";
			triangle_2.position = "-555.798157 -1213.27441 29.0229187";
			triangle_3.position = "-559.747009 -1206.63586 29.0327263";
		]])
		be:reloadStaticCollision();
		reset = false
	end
	
	rotation = rotation+1
	TorqueScript.eval([[
		saw_obj1.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_obj2.rotation = " 1 0 0 ]]..(rotation*2)..[[";
		saw_obj3.rotation = "-1 0 0 ]]..(rotation*3)..[[";
		saw_obj4.rotation = " 1 0 0 ]]..(rotation*4)..[[";
		saw_obj5.rotation = "-1 0 0 ]]..(rotation*5)..[[";
		
		saw_bg_1.rotation =  " 1 0 0 ]].. rotation   ..[[";
		saw_bg_2.rotation =  "-1 0 0 ]]..(rotation*6)..[[";
		saw_bg_3.rotation =  " 1 0 0 ]].. rotation   ..[[";
		saw_bg_4.rotation =  "-1 0 0 ]]..(rotation*6)..[[";
		saw_bg_5.rotation =  " 1 0 0 ]].. rotation   ..[[";
		saw_bg_6.rotation =  " 1 0 0 ]]..(rotation*2)..[[";
		saw_bg_7.rotation =  " 1 0 0 ]].. rotation   ..[[";
		saw_bg_8.rotation =  "-1 0 0 ]]..(rotation*100)..[[";
		saw_bg_9.rotation =  " 1 0 0 ]].. rotation   ..[[";
		saw_bg_10.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_11.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_12.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_13.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_14.rotation = "-1 0 0 ]]..(rotation*3)..[[";
		saw_bg_15.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_16.rotation = " 1 0 0 ]]..(rotation*2)..[[";
		saw_bg_17.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_18.rotation = "-1 0 0 ]]..(rotation*6)..[[";
		saw_bg_19.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_20.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_21.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_22.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_23.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_24.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_25.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_26.rotation = " 1 0 0 ]]..(rotation*7)..[[";
		saw_bg_27.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_28.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_29.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_30.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_31.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_32.rotation = "-1 0 0 ]]..(rotation*3)..[[";
		saw_bg_33.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_34.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_35.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_36.rotation = " 1 0 0 ]]..(rotation*1.5)..[[";
		saw_bg_37.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_38.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_39.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_40.rotation = "-1 0 0 ]]..(rotation*2)..[[";
		saw_bg_41.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_42.rotation = "-1 0 0 ]]..(rotation*4)..[[";
		saw_bg_43.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_44.rotation = "-1 0 0 ]]..(rotation*3)..[[";
		saw_bg_45.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_46.rotation = " 1 0 0 ]]..(rotation*9)..[[";
		saw_bg_47.rotation = " 1 0 0 ]].. rotation   ..[[";
		saw_bg_48.rotation = "-1 0 0 ]]..(rotation*10)..[[";
	]])
end

local function ballsLeft()
	local balls = 0
	for _ in pairs(ballsPotted) do balls = balls+1 end
	
	return 10-balls-ballsLost
end

local function onRaceTick(raceTickTime)
	if start then scenarioTimeOut = scenarioTimeOut+raceTickTime end

	if scenarioTimeOut >= 90 or ballsLeft() == 0 then success(finalScore, finalScore+ballsLost*100, ballsLost*100) end
	
	ui_message('Time Left:  '..math.floor(90-scenarioTimeOut)..'s', 1, 'A')
	ui_message('Balls Left: '..ballsLeft(), 1, 'B')
end

M.onRaceStart = onRaceStart
M.onRaceTick = onRaceTick
M.onBeamNGTrigger = onBeamNGTrigger
M.onPreRender = onPreRender
return M